
from Vector import *

####
# Force Object
# Comp-304 : Winter 2007
#
# A force is composed of a magnitude (vector) which indicate how
# strong and where the force is directed. A start time and an
# end time determines when this force is applied.
#
####

class Force:
       
    ### 
    # Creates a force of a given magnitude (vector), and a given
    # start and end time.
    ###
    def __init__(self, magnitude = Vector(0,0), startTime = 0, endTime = 0):
        
        if type(magnitude) != type(Vector()):
            raise TypeError
        
        if not isinstance(startTime, int):
            raise TypeError
        
        if startTime < 0:
            raise ValueError
        
        if not isinstance(endTime, int):
            raise TypeError

        if endTime < startTime:
            raise ValueError
        
        self.components = []
        
        if endTime != 0:
            self.components.append( (magnitude, startTime, endTime) )
       
    ###
    # Returns a new force which is the composite of this force and the force 
    # provided as an argument.
    ### 
    def add(self, force):
        
        if type(force) != type(self):
            raise TypeError
                
        newForce = Force(Vector(0,0), 0, 0)
        
        for magnitude, startTime, endTime in self.components:
            newForce.components.append( (magnitude, startTime, endTime) )
            
        for magnitude, startTime, endTime in force.components:
            newForce.components.append( (magnitude, startTime, endTime) )
            
        return newForce
            
    ###
    # Returns a new force which is the subtraction of this force to the force 
    # provided as an argument.
    ### 
    def subtract(self, force):
        
        if type(force) != type(self):
            raise TypeError
        
        newForce = Force(Vector(0,0), 0, 0)
        
        for magnitude, startTime, endTime in self.components:
            newForce.components.append( (magnitude, startTime, endTime) )
            
        for magnitude, startTime, endTime in force.components:
            newForce.components.append( (magnitude.opposite(), startTime, endTime) )
            
        return newForce
        
    ###
    # Returns the magnitude at a given time.
    ###
    def getMagnitudeAtTime(self, time):
        
        if not isinstance(time, int):
            raise TypeError
        
        if time < 0:
            raise ValueError
        
        result = Vector()
        
        for magnitude, startTime, endTime in self.components:
            if startTime <= time and endTime >= time:
                result = result.add(magnitude)
                
        return result
    
    ###
    # Returns true if this force is equal to the provided force.
    ###
    def equals(self, force):
        print "You must implement this as part of your assignment."
    
    ###
    # Returns a string representation of this object.
    ###
    def __str__(self):
        strVal = "Force is composed of "
        for magnitude, startTime, endTime in self.components:
            strVal += "\n   Magnitude " + str(magnitude) 
            strVal += " start " + str(startTime) + " end " + str(endTime)
        return strVal